/* performancemonitor.cpp
 *
 * Copyright (C) 2008  Eshwari C
 *
 * Licensed under the Academic Free License version 2.1
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 *
 */
#include "tabdialog.h"

#include <qvbox.h>
#include <qlabel.h>
#include <qdatetime.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qlistbox.h>
#include <qapplication.h>
#include <qtable.h>	
#include<qtimer.h>
#include <qpainter.h>
#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <stdlib.h>
#include <stdio.h>
#include <qdatetime.h>
#include<qnamespace.h>
#include<math.h>
# define PAGES_2K(n,m) BYTES_2K(PAGES_2B(n,m))
# define PAGES_2B(n,m) (unsigned long long) ((n) * m)
# define BYTES_2K(n) (unsigned long long) ((n)>>10)

/**************************************************************************
  This is the constructor which created the performance monitor object. 
  It Creates a dialog which contains three tabs. 
  First tab lists process details.
  Second tab lists the CPU performance parameters.
  Third tab lists the virtual memory parameters.
****************************************************************************/
TabDialog::TabDialog( QWidget *parent, const char *name, const QString &_filename )
    : QTabDialog( parent, name ), filename( _filename ), fileinfo( filename )
{
    setupTab1();/*sets the first tab*/
    setupTab2();/*sets the second tab*/
    setupTab3();/*sets the third tab*/
    	
    arraycu = (int *)malloc(sizeof(int)*55); /*allocate memory for array storing the %CPU for user processes*/
    arraycs = (int *)malloc(sizeof(int)*55); /*allocate memory for array storing the %CPU for sytem processes*/
    arrayci = (int *)malloc(sizeof(int)*55); /*allocate memory for array storing the %CPU for idle processes*/
    arraycw = (int *)malloc(sizeof(int)*55); /*allocate memory for array storing the %CPU for I/O processes*/
    /*allocate memory for array storing page faults*/
    arraypagefaults = (unsigned long long *)malloc(sizeof(unsigned long long)*55);
    /*allocate memory for initial array storing paging ratio*/
    ratio= (double *)malloc(sizeof(unsigned long long)*55);
    /*initialize all the arrays*/
    for(int i=0;i<55;i++)
    {
    	*(arraycu+i)=101;
    	*(arraycs+i)=101;
    	*(arrayci+i)=101;
    	*(arraycw+i)=101;
    	*(arraypagefaults +i)=101;
    	*(ratio+i)=101;
    }
    /*  Fill the first value of %CPU for user,nice,idel,system,I/O waiting processes.
        This values is since boot time */
    QFile statfile("/proc/stat");
    if(statfile.open(IO_ReadOnly))
    {
	unsigned long long cuse,cice,csys,cide,ciow;
//	unsigned long long duse, dsys, didl, diow, Div, divo2;
	QTextStream t( &statfile );        // use a text stream
	QString cmdline;
	while(!t.atEnd())
	{
		cmdline = t.readLine();       // line of text excluding '\n'
		const char *c= (const char *)cmdline;
		if(cmdline.startsWith("cpu0"))
		{		
			//printf("here2");
			sscanf(c,"cpu0  %Lu %Lu %Lu %Lu %Lu", &cuse, &cice, &csys, &cide, &ciow);
			break;
		}
		
	}
	*(arraycu+0)=cuse+cice;
	*(arraycs+0)=csys;
	*(arrayci+0)=cide;
	*(arraycw+0)=ciow;
	}
    /*  Fill the first value of pageins,pageouts and page faults.
        This values is since boot time */

    QFile vmstatfile("/proc/vmstat");
    if(vmstatfile.open(IO_ReadOnly))
    {
	unsigned long long pin,pout,pgfault,pgma,pgmi;
//	unsigned long long duse, dsys, didl, diow, Div, divo2;
	QTextStream t( &vmstatfile );        // use a text stream
	QString cmdline;
	while(!t.atEnd())
	{
		cmdline = t.readLine();       // line of text excluding '\n'
		const char *c= (const char *)cmdline;
		if(cmdline.startsWith("pgmajfault"))
		{
			sscanf(c,"pgmajfault %Lu",&pgma);
		}
		if(cmdline.startsWith("pgfault"))
		{
			sscanf(c,"pgfault %Lu",&pgmi);
		}
		if(cmdline.startsWith("pgpgin"))
		{
			sscanf(c,"pgpgin %Lu",&pin);
		}
		if(cmdline.startsWith("pgpgout"))
		{
			sscanf(c,"pgpgout %Lu",&pout);
		}
		
	}
	*(arraypagefaults+0)=pgma+pgmi;
	
    }
    /*  Set up a times which sends a signal very ten seconds to recalculate all values.
    This timer is also responsible for refreshing the GUI */
    QTimer *timer = new QTimer(this);
    connect( timer, SIGNAL(timeout()),SLOT(timeout()) );
    timer->start( 5000 ); /*send a signal every 10 sec*/
    connect( this, SIGNAL( applyButtonPressed() ), qApp, SLOT( quit() ) );
}
/*******************************************************************************
  This function repaints the GUI. A repaint signal is sent every 10 sec.
  Input - no arguments
  Output - void

********************************************************************************/
void TabDialog::timeout()
{
	repaint(); // sends the repaint signal
}
/********************************************************************************
  The function is called every time a repaint() signal is received.
  All the CPU and memory parameters are collected and computed.
  The GUI is refreshed with new values.
  Input - QPaintevent
  Output - Void

********************************************************************************/


void TabDialog::paintEvent(QPaintEvent *)
{
	/*Clear the contents of each tab*/
	tab2->erase(); 
	tab1->erase();
	tab3->erase();

	/*Update the process table which contains the 
	information about currently running processes in the system*/
	fillTable(ProcessTable);

	/*Update the VM parameters - pageins,pageouts,
	swapins,swapouts,dirty pages,page faults*/
	fillMemoryTable(MemoryTable);
	unsigned long cu,cs,ci,cw;
	unsigned long long pin,pout,pgfault;
	
	/*Calculate %CPU for user,system,idle and IO waiting processes*/
	CPUAggregates(&cu,&cs,&ci,&cw);
	
	/*Calculate Paging ratio and page faults/sec for VM*/
	MemoryAggregates(&pin,&pout,&pgfault);
	
	/*Fill the array with previously computed values which forms the data 
	  for plotting the graph*/
	fillArray(arraycu,cu);
	fillArray(arraycs,cs);
	fillArray(arraycw,cw);
	fillArray(arrayci,ci);
	fillArray(arraypagefaults,pgfault);
	fillArray(ratio,(double)((double)pin/pout));
	
	/*Create the painter for plottin the CPU and Memory performance graph*/
	QPainter paint(tab2,TRUE);
	QPainter paint1(tab3,TRUE);
	
	/*Set brush and pen properties of the graph*/
	paint.setBrush(QBrush(red,SolidPattern));
	paint.setPen(QPen());
	paint1.setBrush(QBrush(red,SolidPattern));
	paint1.setPen(QPen());
	
	/*****Painting the CPU Usage  Tab************/
	/*The CPU usage tab has four graphs and the following lines
	  prints the x and y axis of the graph and then the data points of the graph*/
	
	//1
	/*This plots the %CPU usage for user process graph*/
	paint.drawLine(50,50,50,325); //y axis
	paint.drawLine(50,325,325,325);//x axis
	paint.drawText(70,340,QString("%cpu spent in user processes"),-1,QPainter::LTR);
	drawScale(50,325,20,&paint); //draw the y scale for display
	drawArray(arraycu,50,325,50,325,&paint); //draw the horizontal bar chart
	
	/*This plots the %CPU usage for system process graph*/
	//2
	paint.drawLine(50,375,50,600);//y axis
	paint.drawLine(50,600,325,600);//x axis
	paint.drawText(70,615,QString("%cpu spent in system processes"),-1,QPainter::LTR);	
	drawScale(375,600,20,&paint);//draw the y scale for display
	drawArray(arraycs,50,600,375,600,&paint);//draw the horizontal bar chart
	
	/*This plots the amount of time CPU is idle*/
	//3
	paint.drawLine(375,375,375,600);//y axis
	paint.drawLine(375,600,650,600);//x axis
	paint.drawText(440,615,QString("%cpu spent idle"),-1,QPainter::LTR);	
	drawScale(375,600,350,&paint);//draw the y scale for display
	drawArray(arrayci,375,600,375,600,&paint);//draw the horizontal bar chart
	
	/*This plots the %CPU usage for I/O Waiting process graph*/
	//4
	paint.drawLine(375,50,375,325);//y axis
	paint.drawLine(375,325,650,325);//x axis
	paint.drawText(395,340,QString("%cpu spent in IOWaiting processes"),-1,QPainter::LTR);	
	drawScale(50,325,350,&paint);//draw the y scale for display
	drawArray(arraycw,375,325,50,325,&paint);//draw the horizontal bar chart


	/**Painting the memory tab******/
	/* The memory usage tab prints two graphs pagefaults/sec and paging ration pin/pout*/
	
	//1
	paint1.drawLine(100,100,100,350);//y axis
	paint1.drawLine(100,350,350,350);//x axis
	paint1.drawText(150,370,QString("Page fault rate(per sec)"),-1,QPainter::LTR);	
	drawMemoryScale(100,350,80,&paint1,10);//draw the y scale for display
	drawMemoryArray(arraypagefaults,100,100,350,&paint1);//draw the horizontal bar chart
	
	//2
	paint1.drawLine(400,100,400,350);//y axis
	paint1.drawLine(400,350,650,350);//x axis
	paint1.drawText(450,370,QString("Paging rate(pageins/pageouts)"),-1,QPainter::LTR);	
	drawMemoryScale(100,350,380,&paint1);//draw the y scale for display
	drawMemoryArray(ratio,400,100,350,&paint1);//draw the horizontal bar chart

}

/**********************************************************************************
  This function plots the horizontal bar charts for CPU Usage.
  Input - The data array to be plotted, the origin of the graph, the beginning and end 
          of the y axis so and the painter object
  Output - The graph is plotted and and a void value is returned

*************************************************************************************/
void TabDialog::drawArray(int *arr,int x,int y,int beg,int end,QPainter *paint)
{
	int i;
	double step;
	step = (end-beg)/100;
	for(i=1;i<55;i++)
	{
		if(*(arr+i)==101)
		{
			break;
		}
		else
		{
			int total = (*(arraycu+i)-*(arraycu+i-1))+(*(arraycs+i)-*(arraycs+i-1))+(*(arrayci+i)-*(arrayci+i-1))+(*(arraycw+i)-*(arraycw+i-1));
			double scale = (((double)(*(arr+i)-*(arr+i-1))/total)*100);
			scale = step*scale;
			paint->drawRect(x+5*i,end-scale,5,scale);
		}
	}
	
}
/**********************************************************************************
  This function plots the horizontal bar charts for Memory Usage statistics.
  Input - The data array to be plotted, the origin of the graph, the beginning and end 
          of the y axis so and the painter object
  Output - The graph is plotted and and a void value is returned

*************************************************************************************/

void TabDialog::drawMemoryArray(unsigned long long *arr,int x,int beg,int end,QPainter *paint1)
{
	int i;
	int step=((end-beg));
	for(i=1;i<55;i++)
	{
		if(*(arr+i)==101)
		{
			break;
		}
		else
		{
			int scale = round(((double)(*(arr+i)-*(arr+i-1))/10000));
			scale = round((step*scale)/(double)10);
			paint1->drawRect(x+5*i,(int)(end-scale),5,(int)scale);
		}
	}
	
}

/**********************************************************************************
  This function plots the horizontal bar charts for Memory Usage statistics.  This is 
  an overloaded function.
  Input - The data array to be plotted, the origin of the graph, the beginning and end 
          of the y axis so and the painter object
  Output - The graph is plotted and and a void value is returned

*************************************************************************************/

void TabDialog::drawMemoryArray(double *arr,int x,int beg,int end,QPainter *paint1)
{
	int i;
	double step=(double)((end-beg));
	for(i=1;i<55;i++)
	{
		if(*(arr+i)==101)
		{
			break;
		}
		else
		{
		
		double scale = *(arr+i);
		//printf("value is %f\n",scale);
		scale = step*scale;
		//printf("%f\n",scale);
		//printf("values is %d-%d,total is %d,scale is %f\n",*(arr+i),*(arr+i-1),total,scale);
		paint1->drawRect(x+5*i,(int)(end-scale),5,(int)scale);
		//paint->drawText(x+5*i+1,y+5,QString::number(*(arr+i),10),-1,QPainter::LTR);
		}
	}
	
}
/*********************************************************************************
  This function fills the CPU usage arrays with the appropriate value every tme 
  the data is collected and computed.
 Input - the array to be filled and the value in the particular time instant
 Output - void
 
*********************************************************************************/
void TabDialog::fillArray(int *arr,int value)
{
	int i;
	for(i=0;i<55;i++)
	{
		if(*(arr+i)==101)
		{
			//printf("loop entered: value is %d:i is %d",value,i);
			*(arr+i)=value;
			break;
		}
	}
	if(i==55)
	{
		
		for(int j=0;j<54;j++)
		{
			*(arr+j)=*(arr+j+1);
		}
		*(arr+54) = value;
	}
}

/*********************************************************************************
  This function fills the Memory usage arrays with the appropriate value every tme 
  the data is collected and computed.  This is an overloaded function
 Input - the array to be filled and the value in the particular time instant
 Output - void
 
*********************************************************************************/

void TabDialog::fillArray(unsigned long long *arr,unsigned long long value)
{
	int i;
	for(i=0;i<55;i++)
	{
		if(*(arr+i)==101)
		{
			//printf("loop entered: value is %d:i is %d",value,i);
			*(arr+i)=value;
			break;
		}
	}
	if(i==55)
	{
		
		for(int j=0;j<54;j++)
		{
			*(arr+j)=*(arr+j+1);
		}
		*(arr+54) = value;
	}
}
/*********************************************************************************
  This function fills the Memory usage arrays with the appropriate value every tme 
  the data is collected and computed.  This is an overloaded function
 Input - the array to be filled and the value in the particular time instant
 Output - void
 
*********************************************************************************/

void TabDialog::fillArray(double *arr,double value)
{
	int i;
	for(i=0;i<55;i++)
	{
		if(*(arr+i)==101)
		{
			
			*(arr+i)=value;
			break;
		}
	}
	if(i==55)
	{
		
		for(int j=0;j<54;j++)
		{
			*(arr+j)=*(arr+j+1);
		}
		*(arr+54) = value;
	}
}
/*********************************************************************
  This function is used to draw the values along the Y axis for CPU Usage
  graphs.
  Input - The beginning and end of the y axis and the painter object
  Output - Void
***********************************************************************/
void TabDialog::drawScale(int beg,int end,int x,QPainter *paint)
{
	int step=0;
	step = (end-beg)/10;
	for(int i=0;i<=10;i++)
	{
		paint->drawText(x,end-step*i,QString::number(10*i,10),-1,QPainter::LTR);
	}
}

/***************************************************************************
  This function is used to draw the values along the Y axis for CPU Usage
  graphs.
  Input - The beginning and end of the y axis and the painter object
  Output - Void
*******************************************************************************/

void TabDialog::drawMemoryScale(int beg,int end,int x,QPainter *paint)
{
	int step=0;
	step = (end-beg)/10;
	for(int i=0;i<=10;i++)
	{
		paint->drawText(x,end-step*i,QString::number(0.1*i,'f',1),-1,QPainter::LTR);
	}
}

/********************************************************************************
  This function is used to draw the values along the Y axis for Memory Usage
  graphs.
  Input - The beginning and end of the y axis and the painter object
  Output - Void
*********************************************************************************/

void TabDialog::drawMemoryScale(int beg,int end,int x,QPainter *paint,int units)
{
	int step=0;
	step = (end-beg)/10;
	for(int i=0;i<=10;i++)
	{
		paint->drawText(x,end-step*i,QString::number(i,10),-1,QPainter::LTR);
	}
}
/********************************************************************************
  This function sets up the process details Tab . It creates a table which 
  contains the details of the processes running in the system.
  Input - void
  Output - Void
*********************************************************************************/

void TabDialog::setupTab1()
{
    	tab1 = new QVBox( this );
	tab1->setMargin( 5 );

	ProcessTable = new QTable( tab1, "ProcessTable" );
	ProcessTable->setNumCols( ProcessTable->numCols() + 1 );
    	ProcessTable->horizontalHeader()->setLabel( ProcessTable->numCols() - 1, tr( "Process Name" ) );
    	ProcessTable->setNumCols( ProcessTable->numCols() + 1 );
    	ProcessTable->horizontalHeader()->setLabel( ProcessTable->numCols() - 1, tr( "PID" ) );
    	ProcessTable->setNumCols( ProcessTable->numCols() + 1 );
    	ProcessTable->horizontalHeader()->setLabel( ProcessTable->numCols() - 1, tr( "%MEM" ) );
    	ProcessTable->setNumCols( ProcessTable->numCols() + 1 );
    	ProcessTable->horizontalHeader()->setLabel( ProcessTable->numCols() - 1, tr( "%CPU" ) );
    	ProcessTable->setNumCols( ProcessTable->numCols() + 1 );
    	ProcessTable->horizontalHeader()->setLabel( ProcessTable->numCols() - 1, tr( "State" ) );
    	ProcessTable->setNumCols( ProcessTable->numCols() + 1 );
    	ProcessTable->horizontalHeader()->setLabel( ProcessTable->numCols() - 1, tr( "UID" ) );
    	ProcessTable->setNumCols( ProcessTable->numCols() + 1 );
    	ProcessTable->horizontalHeader()->setLabel( ProcessTable->numCols() - 1, tr( "PPID" ) );
    	ProcessTable->setGeometry( QRect( 40, 40, 410, 460 ) );
    	ProcessTable->setFrameShadow( QTable::Raised );
    	ProcessTable->setVScrollBarMode( QTable::Auto );
    	ProcessTable->setHScrollBarMode( QTable::Auto );
    	ProcessTable->setNumRows( 200 );
    	ProcessTable->setNumCols( 7 );
    	ProcessTable->setSorting( FALSE );
    	addTab( tab1, "Process Details" );
    	fillTable(ProcessTable);
}

/********************************************************************************
  This function sets up the CPU Usage Tab . 
  Input - void
  Output - Void
*********************************************************************************/

void TabDialog::setupTab2()
{
    tab2 = new QVBox(this);
    tab2->setMargin( 5 );
    addTab( tab2, "CPU Usage" );
    
}
/********************************************************************************
  This function sets up the Memory Usage Tab . It creates a table which 
  contains parameters of the virtual memory.
  Input - void
  Output - Void
*********************************************************************************/

void TabDialog::setupTab3()
{
    tab3 = new QVBox( this );
    tab3->setMargin( 5 );
    tab3->setSpacing( 5 );
    
    MemoryTable = new QTable(tab3, "MemoryTable" );    
    MemoryTable->setGeometry(10,10,40,40);    
    MemoryTable->setNumCols( MemoryTable->numCols() + 1 );
    MemoryTable->horizontalHeader()->setLabel( MemoryTable->numCols() - 1, tr( "Dirty Pages" ) );
    MemoryTable->setNumCols( MemoryTable->numCols() + 1 );
    MemoryTable->horizontalHeader()->setLabel( MemoryTable->numCols() - 1, tr( "Swapins" ) );
    MemoryTable->setNumCols( MemoryTable->numCols() + 1 );
    MemoryTable->horizontalHeader()->setLabel( MemoryTable->numCols() - 1, tr( "Swapouts" ) );
    MemoryTable->setNumCols( MemoryTable->numCols() + 1 );
    MemoryTable->horizontalHeader()->setLabel( MemoryTable->numCols() - 1, tr( "Pageins" ) );
    MemoryTable->setNumCols( MemoryTable->numCols() + 1 );
    MemoryTable->horizontalHeader()->setLabel( MemoryTable->numCols() - 1, tr( "Pageouts" ) );
    MemoryTable->setNumCols( MemoryTable->numCols() + 1 );
    MemoryTable->horizontalHeader()->setLabel( MemoryTable->numCols() - 1, tr( "Page Faults" ) );

    MemoryTable->setFrameShadow( QTable::Sunken );
    MemoryTable->setVScrollBarMode( QTable::Auto );
    MemoryTable->setHScrollBarMode( QTable::Auto );

    MemoryTable->setNumRows( 1 );
    MemoryTable->setNumCols( 6 );
    MemoryTable->setSorting( FALSE );
   
    addTab( tab3, "Memory Usage" );
     fillMemoryTable(MemoryTable);
     printf("position is %d,%d",MemoryTable->childX(tab3),MemoryTable->childY(tab3));

}
/********************************************************************************
  This function fills a table in the memory usage tab with performance parameters 
  of the virtual memory. The parameters are dirty pages,pageins,pageouts,swap ins,
  swap outs,page faults. The values are fetched from the /proc/vmstat file
  For details of the file format man proc
  Input - table to be filled
  Output - Void
*********************************************************************************/

void TabDialog::fillMemoryTable(QTable* table)
{
	QFile vmstatfile("/proc/vmstat");
	QFile dirtyfile("dirtyfile.csv");
	char nr_dirty[20],pgpgin[20],pgpgout[20],pgswpin[20],pgswpout[20],pgfault[20],pgmajfault[20];
	int row=1;
	dirtyfile.open(IO_ReadWrite);
	if(vmstatfile.open(IO_ReadOnly))
	{
		QTextStream t( &vmstatfile );        // use a text stream
		QString cmdline;
		QTextStream tf(&dirtyfile);	
		while(!tf.atEnd())
		{
			tf.readLine();
		}

		while(!t.atEnd())
		{
			cmdline = t.readLine();       // line of text excluding '\n'
			const char *c= (const char *)cmdline;
			if(cmdline.startsWith("nr_dirty"))
			{
				sscanf(c,"nr_dirty %s",nr_dirty);
				unsigned long long nd;
				nd = QString(nr_dirty).toULongLong(0,10);
				tf<<(unsigned long)nd;
				tf<<"\n";
				//printf("here dirty pages is %s\n",nr_dirty);
			}
			if(cmdline.startsWith("pgpgin"))
			{
				sscanf(c,"pgpgin %s",pgpgin);
			}
			if(cmdline.startsWith("pgpgout"))
			{
				sscanf(c,"pgpgout %s",pgpgout);
			}
			if(cmdline.startsWith("pswpin"))
			{
				sscanf(c,"pswpin %s",pgswpin);
			}
			if(cmdline.startsWith("pswpout"))
			{
				sscanf(c,"pswpout %s",pgswpout);
			}
			if(cmdline.startsWith("pgmajfault"))
			{
				sscanf(c,"pgmajfault %s",pgmajfault);
			}
			if(cmdline.startsWith("pgfault"))
			{
				sscanf(c,"pgfault %s",pgfault);
			}

		}
	}
	else
	{
		printf("could not open /proc/vmstat");
	}
	table->setItem( 0,0,new QTableItem((QTable*)&table,QTableItem::Always, QString::QString(nr_dirty)));
	table->setItem( 0,1,new QTableItem((QTable*)&table,QTableItem::Always, QString::QString(pgswpin)));
	table->setItem( 0,2,new QTableItem((QTable*)&table,QTableItem::Always, QString::QString(pgswpout)));
	table->setItem( 0,3,new QTableItem((QTable*)&table,QTableItem::Always, QString::QString(pgpgin)));
	table->setItem( 0,4,new QTableItem((QTable*)&table,QTableItem::Always, QString::QString(pgpgout)));
	//table.setItem( row,1,new QTableItem((QTable*)&table,QTableItem::Always, QString::QString(pgfault)));
	unsigned long long totalfault;
	totalfault=QString(pgfault).toULongLong(0,10)+QString(pgmajfault).toULongLong(0,10);
	table->setItem( 0,5,new QTableItem((QTable*)&table,QTableItem::Always, QString::number(totalfault,10)));

}

/********************************************************************************
  This function fills a table in the process details tab with different processes 
  running and statistics about them.The output is similar to ps -ef
  The values are fetched from the /proc.  For details of the file format man proc
  Input - table to be filled
  Output - Void
*********************************************************************************/

void TabDialog::fillTable(QTable* table)
{
    //printf("here1");
    QDir d("/proc");
    d.setSorting( QDir::Name );
    d.setFilter( QDir::Dirs);
    const QFileInfoList *list = d.entryInfoList();
    QFileInfoListIterator it( *list );
    QFileInfo *fi;
    int row =0;
    while ( (fi = it.current()) != 0 ) 
    {
	//printf( "%10li %s\n", fi->size(), fi->fileName().latin1() );
	//printf("here2");
	QString s = fi->fileName();
	bool ok;
	s.toInt(&ok,10);
	if(ok)
	{
	     //table.setItem( row, col,new QTableItem((QTable*)&table,QTableItem::Always, QString::QString(s)));
	    //d.cd(s,FALSE
	    if(d.isReadable())
	    {
		//printf("Readable");
	    }
	    QFile f("/proc/"+s+"/status");
	    //printf("%s",f.name());
	    if(f.open(IO_ReadOnly))
	    {
		//printf("here");
		QTextStream t( &f );        // use a text stream
		QString cmdline;
		while(!t.atEnd())
		{
		    cmdline = t.readLine();       // line of text excluding '\n'
		    //cmdline = cmdline.simplifyWhiteSpace();
		    const char *c= (const char *)cmdline;
		    char *temp = (char*)malloc(sizeof(char)*30);
		    if(cmdline.startsWith("Name"))
		    {		
			sscanf(c,"Name:%s",temp);
			table->setItem( row, 0,new QTableItem(table,QTableItem::Always, QString::QString(temp)));
		    }
		    else if(cmdline.startsWith("Pid"))
		    {
			sscanf(c,"Pid:%s",temp);
			table->setItem( row, 1,new QTableItem(table,QTableItem::Always, QString::QString(temp)));
		    }
		    else if(cmdline.startsWith("PPid"))
		    {
			sscanf(c,"PPid:%s",temp);
			table->setItem( row, 6,new QTableItem(table,QTableItem::Always, QString::QString(temp)));
		    }
		    else if(cmdline.startsWith("Uid"))
		    {
			sscanf(c,"Uid:%s",temp);
			table->setItem( row, 5,new QTableItem(table,QTableItem::Always, QString::QString(temp)));
		    }
		    else if(cmdline.startsWith("State"))
		    {
			sscanf(c,"State:%s",temp);
			table->setItem( row, 4,new QTableItem(table,QTableItem::Always, QString::QString(temp)));
		    }
		}
		
		f.close();
	    }
	    else
	    {
		printf("could not open");
	    }
	    char memusage[20];
	    unsigned long long ret = memory(s);
	    sprintf(memusage,"%Lu",ret);
	    table->setItem( row, 2,new QTableItem((QTable*)&table,QTableItem::Always, QString::QString(memusage)));

	    ++row;
	}
	++it;
    }
 
}

/*************************************************************************
  This function calculates the CPU aggregates. The  % of times CPU is spent 
  in the user processes,system process,being idl and IO-Waitng process
************************************************************************/
void TabDialog::CPUAggregates(unsigned long *cu,unsigned long *cs,unsigned long *ci,unsigned long *cw)
{
	QFile statfile("/proc/stat");
	QFile tempfile("cpuusage.txt");
	tempfile.open(IO_ReadWrite);
	QTextStream tf(&tempfile);
	
	while(!tf.atEnd())
	{
		tf.readLine();
	}
	if(statfile.open(IO_ReadOnly))
	{
		unsigned long long cuse,cice,csys,cide,ciow;
		QTextStream t( &statfile );        // use a text stream
		QString cmdline;
		QDateTime time;
		time = QDateTime::currentDateTime();
		while(!t.atEnd())
		{
			cmdline = t.readLine();       // line of text excluding '\n'
			const char *c= (const char *)cmdline;
			if(cmdline.startsWith("cpu0"))
			{		
				//Cpu Time in User Processes
				sscanf(c,"cpu0  %Lu %Lu %Lu %Lu %Lu", &cuse, &cice, &csys, &cide, &ciow);
				*cu = (unsigned long)cuse+(unsigned long)cice; 
				tf<<(unsigned long)cuse;
				tf<<"\t";
				tf<<(unsigned long)cice;
				tf<<"\t";
				//Cpu Time in Idle Processes				
				*ci = cide; 
				tf<<(unsigned long)cide;
				tf<<"\t";
				*cw = ciow; 
				//Cpu Time in IOWaiting Processes
				tf<<(unsigned long)ciow;
				tf<<"\t";
				//Cpu Time in  System Processes				
				*cs = csys;
				tf<<(unsigned long)csys;
				tf<<"\t";
				tf<<"\n";
					
			}
		}

	}
	else
	{
		printf("could not open /proc/stat file");
	}
}
/*************************************************************************
  This function calculates the Memory aggregates. It calculates the number of page
  ins,page outs and page faults.
************************************************************************/

void TabDialog::MemoryAggregates(unsigned long long *pin,unsigned long long *pout,unsigned long long *pfault)
{
	QFile vmstatfile("/proc/vmstat");
	QFile MemAggfile("memoryusage.csv");
	char pgfault[20],pgmajfault[20],pgpgin[20],pgpgout[20];
	MemAggfile.open(IO_ReadWrite);
	QTextStream tf(&MemAggfile);
	
	while(!tf.atEnd())
	{
		tf.readLine();
	}

	if(vmstatfile.open(IO_ReadOnly))
	{
		QTextStream t( &vmstatfile );        // use a text stream
		QString cmdline;
		
		while(!t.atEnd())
		{
			cmdline = t.readLine();       // line of text excluding '\n'
			const char *c= (const char *)cmdline;
			if(cmdline.startsWith("pgmajfault"))
			{
				sscanf(c,"pgmajfault %s",pgmajfault);
			}
			if(cmdline.startsWith("pgfault"))
			{
				sscanf(c,"pgfault %s",pgfault);
			}
			if(cmdline.startsWith("pgpgin"))
			{
				sscanf(c,"pgpgin %s",pgpgin);
			}
			if(cmdline.startsWith("pgpgout"))
			{
				sscanf(c,"pgpgout %s",pgpgout);
			}
		}
	}
	else
	{
		printf("could not open /proc/vmstat");
	}
	
	unsigned long long totalfault;
	totalfault=QString(pgfault).toULongLong(0,10)+QString(pgmajfault).toULongLong(0,10);
	*(pfault) = totalfault;
	*pin= QString(pgpgin).toLongLong(0,10);
	*pout= QString(pgpgout).toLongLong(0,10);
	tf<<(unsigned long)*pin;
	tf<<";";
	tf<<(unsigned long)*pout;
	tf<<";";
	tf<<(unsigned long)*pfault;
	tf<<";";
	tf<<"\n";


}
/*************************************************************************
  This function calculates the %MEM memory usage for each process
  Input - the process id
  Output - The %MEM usage of that process.
************************************************************************/


unsigned long long TabDialog::memory(QString num)
{

unsigned long long size,resident,blah,cide,ciow,pagesize;
unsigned long long temp;
//char s[40];		

//printf("Enter the filename:");
//scanf("%s",s);

QFile f("/proc/"+num+"/statm");
//QFile f(s);
	if(f.open(IO_ReadOnly))
	{
	    QTextStream t( &f );        // use a text stream
	    QString cmdline;
	
            while(!t.atEnd())
	    {
	         cmdline = t.readLine();       // line of text excluding '\n'
	         const char *c= (const char *)cmdline;
	         sscanf(c,"%Lu %Lu %Lu %Lu %Lu", &size, &resident, &blah, &cide, &ciow);
	    }
        }
	else
	{
		printf("Can't open file\n");
	}
	
QFile x("/proc/meminfo");
	if(x.open(IO_ReadOnly))
	{
	    QTextStream t( &x );        // use a text stream
	    QString cmdline;
	    while(!t.atEnd())
	    {
	         cmdline = t.readLine();       // line of text excluding '\n'
	         //cmdline = cmdline.simplifyWhiteSpace();
	         const char *c= (const char *)cmdline;

	         
                 if(cmdline.startsWith("MemTotal"))
		 {		
		     sscanf(c,"MemTotal:%Lu",&temp);
	         }
	    }
	}
	else
	{
		printf("Can't open file\n");
	}

	
	pagesize = getpagesize();
    unsigned long long value;
	//printf("%Lu",PAGES_2K(100*resident/temp,pagesize));
     value = PAGES_2K(100*resident/temp,pagesize);
  //   printf("value is :%Lu\n",value);
	f.close();
	
return value;
}
